SystemOrganization addCategory: #MECCG!
SystemOrganization addCategory: 'MECCG-Model'!
SystemOrganization addCategory: 'MECCG-Model-Tests'!

TestCase subclass: #ModelUsageExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!ModelUsageExamples methodsFor: 'as yet unclassified' stamp: 'SergioFedi 4/17/2016 17:42'!
test01SimpleUsage
	"Modelar empezar un juego con N jugadores que empieza (Game Setup), 
	se juegan turnos y termina (Victory Condition)"
	| match result |

	match := Match of: MiddleEarthCardGame rules.
	match addPlayer: (Player named: 'Fede Padawan').
	match addPlayer: (Player named: 'Pieter').
	self assert: match result isNil.
	match setup.
	self assert: match result notNil.
	self assert: match isReady.
	self assert: match result matchDuration isNil.
	match play.
	match wrapUp.
	result := match result.
	self assert: result matchDuration notNil.
! !

Object subclass: #Match
	instanceVariableNames: 'rules players result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Match commentStamp: 'SergioFedi 4/21/2016 08:22' prior: 0!
I model a match of a game.

You can see how to use me in ModelUsageExamples
!

!Match class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:12'!
of: matchRules

	^self new
		rules: matchRules.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:18'!
addPlayer: aPlayer

	players add: aPlayer.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:19'!
initialize
	super initialize.
	players := OrderedCollection new.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:33'!
isReady

	^self rules isMatchReady: self.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:37'!
play

	self rules play: self.
! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:48'!
players
	^ players! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:40'!
result

	^result.! !

!Match methodsFor: 'accesing' stamp: 'SergioFedi 4/17/2016 17:49'!
result: aMatchResult

	result := aMatchResult.! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:13'!
rules
	^ rules! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:13'!
rules: anObject
	rules := anObject! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:25'!
setup

	(self rules isMatchReadyForSetup: self)
		ifTrue: [ self rules setupMatch: self ].! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUp

	self rules wrapUpMatch: self.! !

Object subclass: #MatchOutcome
	instanceVariableNames: 'matchStatistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MatchOutcome commentStamp: 'SergioFedi 4/21/2016 08:27' prior: 0!
I model the outcome of a match.

The particular data I will contain will be determined by the rules of the match.
!

!MatchOutcome methodsFor: 'as yet unclassified' stamp: 'SergioFedi 4/22/2016 08:59'!
initialize
	super initialize.
	matchStatistics := Dictionary new.
	self matchDuration: nil.
	self isFinished: false.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished

	^matchStatistics at: 'isFinished'.! !

!MatchOutcome methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished: anObject 

	matchStatistics at: 'isFinished' put: anObject.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'SergioFedi 4/17/2016 17:50'!
matchDuration

	^matchStatistics at: 'matchDuration'.! !

!MatchOutcome methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:56'!
matchDuration: anObject 

	matchStatistics at: 'matchDuration' put: anObject.! !

Object subclass: #MiddleEarthCardGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGame commentStamp: 'SergioFedi 4/21/2016 08:31' prior: 0!
I model the game of Middle Earth Collectible Card Game.
I can answer the rules of the game.
!

!MiddleEarthCardGame class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:09'!
rules

	^MiddleEarthCardGameRules new.! !

Object subclass: #MiddleEarthCardGameRules
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGameRules commentStamp: 'SergioFedi 4/21/2016 08:36' prior: 0!
I model the rules of the Middle Earth Card Game.

My responsibility is to run matches of that game.
!

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'SergioFedi 4/22/2016 09:03'!
incrementRoundCountOf: aMatch

	aMatch result matchDuration: aMatch result matchDuration + 1.! !

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'SergioFedi 4/22/2016 09:05'!
initializeRoundCountOf: aMatch 

	aMatch result matchDuration: 0.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 09:01'!
isFinished: aMatch

	^aMatch result isFinished.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:34'!
isMatchReady: aMatch

	^true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:31'!
isMatchReadyForSetup: aMatch

	^true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 09:04'!
play: aMatch
	"Play the match until it is finished"

	self initializeRoundCountOf: aMatch.
	[true] whileTrue: [
			self playRoundOf: aMatch.
			self incrementRoundCountOf: aMatch.
			(self isFinished: aMatch) ifTrue: [ ^ self ].
		].
! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:42'!
playRoundOf: aMatch
	aMatch players
		do: [ :player | 
			self playTurnOf: player in: aMatch.
			(self isFinished: aMatch)
				ifTrue: [ ^ self ] ]! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 09:00'!
playTurnOf: aPlayer in: aMatch

	((1 to: 10) atRandom > 8) ifTrue: [ 
		aMatch result isFinished: true.
	]! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/21/2016 08:28'!
setupMatch: aMatch

	aMatch result: MatchOutcome new.
	^self.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUpMatch: aMatch

	^self.! !

Object subclass: #Player
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Player commentStamp: 'SergioFedi 4/21/2016 08:30' prior: 0!
I model a participant of a Match that plays in it.
!

!Player class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:16'!
named: aString

	^self new name: aString.! !

!Player methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:15'!
name
	^ name! !

!Player methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:15'!
name: anObject
	name := anObject! !
